#include "nandflash.h"

uint8_t InvalidBlockTable[NANDFLASH_NUMOF_BLOCK];

/*********************************************************************//**
 * @brief 		Ready/Busy check, no timeout, basically, R/B bit should
 * 				once to bail out from this routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void NandFlash_WaitForReady( void )
{
//	uint8_t i;

	while((LPC_GPIO_PORT->PIN[6] & (1<<13)) )
	while(!(LPC_GPIO_PORT->PIN[6] & (1<<13)) )	
	
	return;
}
/*********************************************************************//**
 * @brief 		Configure the byte lane state
 * @param[in]	index index number, should be from 0 to 3
 * @param[in] 	pb_val  Byte lane state, should be:
 * 					- 0: For reads all bits in BLSn[3:0] are HIGH.
 * 					- 1: For reads all bits in BLSn[3:0] are LOW.
 * @return 		None
 **********************************************************************/
void EMC_StaMemConfigPB(uint32_t index , uint32_t pb_val)
{
   uint32_t mask =  ~(uint32_t)(_BIT(7))  ;
   switch ( index)
   {
      case 0:
         LPC_EMC->STATICCONFIG0 = (LPC_EMC->STATICCONFIG0  & mask)| pb_val;
	   break;
      case 1:
         LPC_EMC->STATICCONFIG1 = (LPC_EMC->STATICCONFIG1  & mask)| pb_val;
	   break;
      case 2:
         LPC_EMC->STATICCONFIG2 =( LPC_EMC->STATICCONFIG2  & mask)| pb_val;
	   break;
      case 3:
         LPC_EMC->STATICCONFIG3 = (LPC_EMC->STATICCONFIG3  & mask)| pb_val;
	   break;
   }
}
/*********************************************************************//**
 * @brief 		Configure the memory width
 * @param[in]	index index number, should be from 0 to 3
 * @param[in] 	mem_width  memory width, should be:
 * 					- 0x00: 8-bits
 * 					- 0x01: 16-bits
 * 					- 0x02: 32-bits
 * @return 		None
 **********************************************************************/
void EMC_StaMemConfigMW(uint32_t index , uint32_t mem_width)
{
   uint32_t mask =  ~(uint32_t)(0x03)  ;
   switch ( index)
   {
      case 0:
         LPC_EMC->STATICCONFIG0 = (LPC_EMC->STATICCONFIG0 & mask) | mem_width;
	   break;
      case 1:
         LPC_EMC->STATICCONFIG1 = (LPC_EMC->STATICCONFIG1 & mask) | mem_width;
	   break;
      case 2:
         LPC_EMC->STATICCONFIG2 = (LPC_EMC->STATICCONFIG2 & mask)| mem_width;
	   break;
      case 3:
         LPC_EMC->STATICCONFIG3 = (LPC_EMC->STATICCONFIG3 & mask) | mem_width;
	   break;
   }
}

/*********************************************************************//**
 * @brief 		Set the value of LPC_EMC static memory registers
 * @param[in]	index index number, should be from 0 to 3
 * @param[in] 	EMC_STA_MEM_PAR  Static register, should be:
 * 					- EMC_STA_MEM_WAITWEN: StaticWaitWen0 register
 * 					- EMC_STA_MEM_WAITOEN: StaticWaitOen0 register
 * 					- EMC_STA_MEM_WAITRD: StaticWaitRd0 register
 * 					- EMC_STA_MEM_WAITPAGE: StaticWaitPage0 register
 * 					- EMC_STA_MEM_WAITWR: StaticWaitWr0 register
 * 					- EMC_STA_MEM_WAITTURN: StaticWaitTurn0 register
 * @return 		None
 **********************************************************************/
void EMC_SetStaMemoryParameter(EMC_STA_MEM_PAR par, uint32_t val)
{

		 switch ( par)
		 {
				case EMC_STA_MEM_WAITWEN:
					 LPC_EMC->STATICWAITWEN1 = val;
					 break;
				case EMC_STA_MEM_WAITOEN:
					 LPC_EMC->STATICWAITOEN1 = val;
					 break;
				case EMC_STA_MEM_WAITRD:
					 LPC_EMC->STATICWAITRD1 = val;
					 break;
				case EMC_STA_MEM_WAITPAGE:
					 LPC_EMC->STATICWAITPAG1 = val;
					 break;   
				case EMC_STA_MEM_WAITWR:
					 LPC_EMC->STATICWAITWR1 = val;
					 break;
				case EMC_STA_MEM_WAITTURN:
					 LPC_EMC->STATICWAITTURN1 = val;
					 break;
		 }

}


 void NandFlash_Init(void)
{
	uint32_t i;
	TIM_TIMERCFG_Type TIM_ConfigStruct;
	
  /* DATA LINES 0..31 > D0..D31 */
  	/* P1_7 - EXTBUS_D0 ?External memory data line 0 */
    scu_pinmux(0x1,  7,  (MD_PLN | MD_EZI | MD_ZI | MD_EHS), FUNC3);  /* P1_7: D0 (function 0) errata */
    scu_pinmux(0x1,  8,  (MD_PLN | MD_EZI | MD_ZI | MD_EHS), FUNC3);  /* P1_8: D1 (function 0) errata */
    scu_pinmux(0x1,  9,  (MD_PLN | MD_EZI | MD_ZI | MD_EHS), FUNC3);  /* P1_9: D2 (function 0) errata */
    scu_pinmux(0x1,  10, (MD_PLN | MD_EZI | MD_ZI | MD_EHS), FUNC3);  /* P1_10: D3 (function 0) errata */
    scu_pinmux(0x1,  11, (MD_PLN | MD_EZI | MD_ZI | MD_EHS), FUNC3);  /* P1_11: D4 (function 0) errata */
    scu_pinmux(0x1,  12, (MD_PLN | MD_EZI | MD_ZI | MD_EHS), FUNC3);  /* P1_12: D5 (function 0) errata */
    scu_pinmux(0x1,  13, (MD_PLN | MD_EZI | MD_ZI | MD_EHS), FUNC3);  /* P1_13: D6 (function 0) errata */
    scu_pinmux(0x1,  14, (MD_PLN | MD_EZI | MD_ZI | MD_EHS), FUNC3);  /* P1_14: D7 (function 0) errata */	
	
	scu_pinmux(	6 ,	7	,	(MD_PLN | MD_EZI | MD_ZI | MD_EHS)	,	FUNC1	);	//CLE -EMC_A15
	scu_pinmux(	0X0A	,	4	,	(MD_PLN | MD_EZI | MD_ZI | MD_EHS)	,	FUNC3	);	//ALE -EMC_A23
	
	scu_pinmux(	1	,	6	,	(MD_PLN | MD_EZI | MD_ZI | MD_EHS)	,	FUNC3	);	//WE	
	scu_pinmux(	1	,	3	,	(MD_PLN | MD_EZI | MD_ZI | MD_EHS)	,	FUNC3	);	//OE	

	scu_pinmux(	6	,	3	,	(MD_PLN | MD_EZI | MD_ZI | MD_EHS)	,	FUNC3	);	//CS1


	
	scu_pinmux(	0X0C	,	14	,	MD_PLN_FAST	,	FUNC4	);	//NAND_R/B
	GPIO_SetDir(6,(1<<13), 0);  //- 0: Input.

		/* Set up EMC Controller */

	LPC_EMC->STATICWAITRD1 = 0x06;
	LPC_EMC->CONTROL = 0x01;
	LPC_EMC->STATICCONFIG1 = 0x80;
	LPC_EMC->STATICWAITOEN1 = 0x01;
	

		
	EMC_SetStaMemoryParameter(EMC_STA_MEM_WAITWEN, EMC_StaticWaitWen_WAITWEN(2));

	EMC_SetStaMemoryParameter(EMC_STA_MEM_WAITOEN, EMC_StaticWaitOen_WAITOEN(2));

	EMC_SetStaMemoryParameter(EMC_STA_MEM_WAITWR, EMC_StaticWaitwr_WAITWR(0x0f));

	EMC_SetStaMemoryParameter(EMC_STA_MEM_WAITPAGE, EMC_StaticwaitPage_WAITPAGE(0x0f));

	EMC_SetStaMemoryParameter(EMC_STA_MEM_WAITWR, EMC_StaticWaitwr_WAITWR(0x0f));

	EMC_SetStaMemoryParameter(EMC_STA_MEM_WAITTURN, EMC_StaticWaitTurn_WAITTURN(0x0f));
	
	
	    // init timer
	TIM_ConfigStruct.PrescaleOption = TIM_PRESCALE_USVAL;
	TIM_ConfigStruct.PrescaleValue	= 1;
		// Set configuration for Tim_config and Tim_MatchConfig
	TIM_Init(LPC_TIMER0, TIM_TIMER_MODE,&TIM_ConfigStruct);

  	// wait 2ms
	TIM_Waitms(20);
	
	/* assume all blocks are valid to begin with */
	for ( i = 0; i < NANDFLASH_NUMOF_BLOCK; i++ )
	{
		InvalidBlockTable[i] = 0;
		TIM_Waitus(2);
	}
}


/*********************************************************************//**
 * @brief 		Issue Reset command to NAND FLASH memory
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void NandFlash_Reset( void )
{
	volatile uint8_t *pCLE;

	/* Reset NAND FLASH  through NAND FLASH command */
	pCLE = K9F1G_CLE;
	*pCLE = K9FXX_RESET;

	TIM_Waitms(2);
	return;
}

/*********************************************************************//**
 * @brief 		Read status from NAND FLASH memory
 * @param[in]	Cmd	command for read operation, should be:
 * 					-  K9FXX_BLOCK_PROGRAM_1
 *					-  K9FXX_BLOCK_ERASE_1
 *					-  K9FXX_READ_1
 * @return 		Status, could be:
 *				- TRUE: pass
 *				- FALSE: Failure
 **********************************************************************/
Bool NandFlash_ReadStatus(uint32_t Cmd)
{
	volatile uint8_t *pCLE;
	volatile uint8_t *pDATA;
	uint8_t StatusData;

	pCLE  = K9F1G_CLE;
	pDATA = K9F1G_DATA;

	*pCLE = K9FXX_READ_STATUS;

#if (_CUR_USING_NANDFLASH == _RUNNING_NANDFLASH_K9F1G08U0C)
	while ( (*pDATA & 0xC0) != 0xC0 );
#else
	/* Wait until bit 5 and 6 are 1, READY, bit 7 should be 1 too, not protected */
	/* if ready bit not set, it gets stuck here */
	while ( (*pDATA & 0xE0) != 0xE0 );
#endif

	StatusData = *pDATA;

	switch (Cmd)
	{
		case K9FXX_BLOCK_PROGRAM_1:
		case K9FXX_BLOCK_ERASE_1:
		  	if (StatusData & 0x01)	/* Erase/Program failure(1) or pass(0) */
				return(FALSE);
		  	else
				return(TRUE);

		case K9FXX_READ_1:				/* bit 5 and 6, Read busy(0) or ready(1) */
		  	return(TRUE);

		default:
		  	break;
	}

	return(FALSE);
}
/*********************************************************************//**
 * @brief 		Read ID from external NAND FLASH memory
 * @param[in]	None
 * @return 		ID value
 **********************************************************************/
uint32_t NandFlash_ReadId( void )
{
	uint8_t a=0, b=0, c=0, d=0;
	
	volatile uint8_t *pCLE;
	volatile uint8_t *pALE;
	volatile uint8_t *pDATA;

	pCLE  = K9F1G_CLE;
	pALE  = K9F1G_ALE;
	pDATA = K9F1G_DATA;

	*pCLE = K9FXX_READ_ID;
	*pALE = 0;

	a = *pDATA;
	b = *pDATA;
	d = *pDATA;
	c = *pDATA;
	
//	printf("a = 0X%x\r\n",a);
//	printf("b = 0X%x\r\n",b);
// 	printf("c = 0X%x\r\n",c);
// 	printf("d = 0X%x\r\n",d);
	
	return ((a << 24) | (b << 16) | (c << 8) | d);
}

/*********************************************************************//**
 * @brief 		Erase the whole NAND FLASH memory block based on the
 *				block number
 * @param[in]	blockNum	number of block that will be erased, should
 *				be in range: 0 .. 1023
 * @return 		Erase status, could be:
 * 					- TRUE: pass
 *					- FALSE: failure
 **********************************************************************/
Bool NandFlash_BlockErase( uint32_t blockNum )
{
	volatile uint8_t *pCLE;
	volatile uint8_t *pALE;
	uint32_t rowAddr;

	pCLE  = K9F1G_CLE;
	pALE  = K9F1G_ALE;

	rowAddr = (NANDFLASH_BASE_ADDR + blockNum * NANDFLASH_BLOCK_FSIZE);
	rowAddr = rowAddr - (rowAddr % NANDFLASH_BLOCK_FSIZE);

	*pCLE = K9FXX_BLOCK_ERASE_1;

	*pALE = (uint8_t)(rowAddr & 0x00FF);			/* column address low */

	*pALE = (uint8_t)((rowAddr & 0xFF00) >> 8);	/* column address high */

	*pCLE = K9FXX_BLOCK_ERASE_2;

	NandFlash_WaitForReady();

	return(NandFlash_ReadStatus(K9FXX_BLOCK_ERASE_1));
}

/*********************************************************************//**
 * @brief 		This routine is used to check if the block is valid or
 *				not.
 * @param[in]	None
 * @return 		Checking status, could be:
 * 					- TRUE: all blocks are valid
 *					- FALSE: invalid block is found, an initial invalid
 *						     table will be created
 **********************************************************************/
Bool NandFlash_ValidBlockCheck( void )
{
 	volatile uint8_t *pCLE;
	volatile uint8_t *pALE;
 	volatile uint8_t *pDATA;
	uint32_t block, page;
	Bool retValue = TRUE;

	uint8_t data = 0;

	pCLE  = K9F1G_CLE;
	pALE  = K9F1G_ALE;
	pDATA = K9F1G_DATA;

	for ( block = 0; block < NANDFLASH_NUMOF_BLOCK; block++ )
	{
		for ( page = 0; page < 2; page++ )
		{
			/* Check column address 2048 at first page and second page */
			NandFlash_PageReadFromAddr(block, page, NANDFLASH_INVALIDBLOCK_CHECK_COLUMM, &data, 1);
			
			if(data != 0xFF)
			{
				// found invalid block number, mark block number in the invalid
				// block table
				InvalidBlockTable[block] = 1;

				//At least one block is invalid
				retValue = FALSE;
			}
		}
	}

	return(retValue);
}

/*********************************************************************//**
 * @brief 		Write a full page of program into NAND flash based on the
 *				page number, write up to 2112 bytes of data at a time.
 * @param[in]	pageNum		number of page that will be programmed, should
 * 				be in range: 0..63
 * @param[in]	blockNum	number of block that will be programmed, should
 * 				be in range: 0..1023
 * @param[in]	bufPtr		pointer to the buffer that contain data will be
 * 				programmed in flash memory
 * @return 		Program status, could be:
 * 					- TRUE: success
 * 					- FALSE: fail
 **********************************************************************/
Bool NandFlash_PageProgram( uint32_t pageNum, uint32_t blockNum, uint8_t *bufPtr )
{
	volatile uint8_t *pCLE;
	volatile uint8_t *pALE;
	volatile uint8_t *pDATA;
	uint32_t i, curAddr, curColumm;

	pCLE  = K9F1G_CLE;
	pALE  = K9F1G_ALE;
	pDATA = K9F1G_DATA;

	curAddr = NANDFLASH_BASE_ADDR + blockNum * NANDFLASH_BLOCK_FSIZE
								+ pageNum * NANDFLASH_PAGE_FSIZE;

	curColumm = curAddr % NANDFLASH_PAGE_FSIZE;
	curAddr -= curColumm;

	*pCLE = K9FXX_BLOCK_PROGRAM_1;

	*pALE =  (uint8_t)(curColumm & 0x000000FF);		/* column address low */

	*pALE = (uint8_t)((curColumm & 0x00000F00) >> 8);	/* column address high */

	*pALE = (uint8_t)((curAddr & 0x00FF0000) >> 16);	/* row address low */

	*pALE = (uint8_t)((curAddr & 0xFF000000) >> 24);	/* row address high */

	//Not write to spare area for the NandFlash valid block checking
	for ( i = 0; i < NANDFLASH_RW_PAGE_SIZE; i++ )
	{
		*pDATA = *bufPtr++;
	}

	*pCLE = K9FXX_BLOCK_PROGRAM_2;

	NandFlash_WaitForReady();

	return( NandFlash_ReadStatus( K9FXX_BLOCK_PROGRAM_1 ) );
}

/*********************************************************************//**
 * @brief 		Read the whole NAND FLASH memory page based on the
 *				page number, the data will be stored in the pointer
 *				to the buffer.
 * @param[in]	pageNum		number of page that will be read, should
 * 				be in range: 0..63
 * @param[in]	blockNum	number of block that will be read, should
 * 				be in range: 0..1023
 * @param[in]	bufPtr		pointer to the buffer that contain data will be
 * 				read from flash memory
 * @return 		Read status, could be:
 * 					- TRUE: success
 * 					- FALSE: fail
 **********************************************************************/
Bool NandFlash_PageRead( uint32_t pageNum, uint32_t blockNum, uint8_t *bufPtr )
{
	return (NandFlash_PageReadFromBeginning(pageNum, blockNum, bufPtr) != 0);
}

/*********************************************************************//**
 * @brief 		Read the whole NAND FLASH memory page based on the
 *				page number, the data will be stored in the pointer
 *				to the buffer.
 * @param[in]	pageNum		number of page that will be read, should
 * 				be in range: 0..63
 * @param[in]	blockNum	number of block that will be read, should
 * 				be in range: 0..1023
 * @param[in]	bufPtr		pointer to the buffer that contain data will be
 * 				read from flash memory
 * @return 		number of byte(s) read til the end of the page
 **********************************************************************/
int NandFlash_PageReadFromBeginning(uint32_t blockNum, uint32_t pageNum, uint8_t* bufPtr)
{
	return (NandFlash_PageReadFromAddr(blockNum, pageNum, 0, bufPtr, NANDFLASH_PAGE_FSIZE));
}

/*********************************************************************//**
 * @brief 		Read the whole NAND FLASH memory page based on the
 *				page number, the data will be stored in the pointer
 *				to the buffer.
 * @param[in]	blockNum	number of block that will be read, should
 * 							be in range: 0..1023
 * @param[in]	pageNum		number of page that will be read, should
 * 				be in range: 0..63
 * @param[in]	addrInPage	the address in NandFlash to be read,
 * 							calculated from the beginning of page
 * @param[in]	bufPtr		pointer to the buffer that contain data will be
 * 							read from flash memory
 * @param[in]	size	the number of byte(s) to be read and stored to the buffer
 * @return 		number of byte(s) read til the end of the page
 **********************************************************************/
int NandFlash_PageReadFromAddr(uint32_t blockNum, uint32_t pageNum,
											uint32_t addrInPage, uint8_t* bufPtr, uint32_t size)
{
	uint32_t curAddr = 0;

	curAddr += NANDFLASH_BASE_ADDR + blockNum * NANDFLASH_BLOCK_FSIZE;
	TIM_Waitus(1);
	curAddr += pageNum * NANDFLASH_PAGE_FSIZE;
	TIM_Waitus(1);
	curAddr += addrInPage;
	TIM_Waitus(1);
	return (NandFlash_ReadFromAddr(curAddr, bufPtr, size));
}

/*********************************************************************//**
 * @brief 		Read the whole NAND FLASH memory at an expected address,
 *				the data will be stored in the pointer to the buffer.
 * @param[in]	addrInWholeNand	the address in NandFlash to be read,
 * 						calculated from the beginning of Nand base address 0
 * @param[in]	bufPtr	pointer to the buffer that contain data will be
 * 						read from flash memory
 * @param[in]	size	the number of byte(s) to be read and stored to the buffer
 * @return 		number of byte(s) read til the end of the page
 **********************************************************************/
int NandFlash_ReadFromAddr(uint32_t addrInWholeNand, uint8_t* bufPtr, uint32_t size)
{
	volatile uint8_t *pCLE;
	volatile uint8_t *pALE;
	volatile uint8_t *pDATA;
	uint32_t i, curColumm, curRow;

	i = 0;

	pCLE  = K9F1G_CLE;
	pALE  = K9F1G_ALE;
	pDATA = K9F1G_DATA;

	curColumm = addrInWholeNand % NANDFLASH_PAGE_FSIZE;
	curRow = addrInWholeNand - curColumm;

	*pCLE = K9FXX_READ_1;

	*pALE = (uint8_t)(curColumm & 0x000000FF);			/* column address low */

	*pALE = (uint8_t)((curColumm & 0x00000F00) >> 8);		/* column address high */

	*pALE = (uint8_t)((curRow & 0x00FF0000) >> 16);	/* row address low */

	*pALE = (uint8_t)((curRow & 0xFF000000) >> 24);	/* row address high */

	*pCLE = K9FXX_READ_2;

	NandFlash_WaitForReady();

	//Get data from the current address in the page til the end of the page
	for ( i = 0; i < (NANDFLASH_PAGE_FSIZE - curColumm); i++ )
	{
		*bufPtr = *pDATA;

		bufPtr++;

		if((i + 1) >= size)
			break;
	}

	// Ok, return
	return i;
}

