/**********************************************************************
* $Id$		Gpio_LedBlinky.c	2011-06-02
*//**
* @file		Gpio_LedBlinky.c
* @brief	This example describes how to use GPIO to drive LEDs
* @version	1.0
* @date		02. June. 2011
* @author	NXP MCU SW Application Team
*
* Copyright(C) 2011, NXP Semiconductor
* All rights reserved.
*
***********************************************************************
* Software that is described herein is for illustrative purposes only
* which provides customers with programming information regarding the
* products. This software is supplied "AS IS" without any warranties.
* NXP Semiconductors assumes no responsibility or liability for the
* use of the software, conveys no license or title under any patent,
* copyright, or mask work right to the product. NXP Semiconductors
* reserves the right to make changes in the software without
* notification. NXP Semiconductors also make no representation or
* warranty that such application will be suitable for the specified
* use without further testing or modification.
**********************************************************************/
#include "lpc43xx_gpio.h"
#include "lpc43xx_scu.h"
#include "lpc43xx_libcfg_default.h"
#include "lpc43xx_cgu.h"
#include "LPC4357_IO.h"
#include "lpc4357_uart.h"
#include "nandflash.h"
#include "lpc43xx_emc.h"

//#include "lpc43xx_utils.h"


/* Example group ----------------------------------------------------------- */
/** @defgroup Gpio_LedBlinky	Gpio_LedBlinky
 * @ingroup GPIO_Examples
 * @{
 */

uint8_t ReadBuf[NANDFLASH_PAGE_FSIZE], WriteBuf[NANDFLASH_PAGE_FSIZE];
extern uint8_t InvalidBlockTable[NANDFLASH_NUMOF_BLOCK];

/************************** PRIVATE VARIABLES *************************/
uint32_t msec;


/* With ARM and GHS toolsets, the entry point is main() - this will
   allow the linker to generate wrapper code to setup stacks, allocate
   heap area, and initialize and copy code and data segments. For GNU
   toolsets, the entry point is through __start() in the crt0_gnu.asm
   file, and that startup code will setup stacks and data */
int main(void)
{
	uint32_t i;
	uint32_t FlashID;
  SystemInit();
	
	//	CGU_Init();
	
	CGU_SetXTALOSC(12000000);
	CGU_EnableEntity(CGU_CLKSRC_XTAL_OSC, ENABLE);
	CGU_EntityConnect(CGU_CLKSRC_XTAL_OSC, CGU_CLKSRC_PLL1);
	// Disable PLL1 CPU hang???
	//CGU_EnableEntity(CGU_CLKSRC_PLL1, DISABLE);
	CGU_SetPLL1(10);
	CGU_EnableEntity(CGU_CLKSRC_PLL1, ENABLE);
	CGU_EntityConnect(CGU_CLKSRC_PLL1, CGU_BASE_M4);
	CGU_UpdateClock();	
	printf("Core is clocked at %1.1f MHz\r\n", ((float)CGU_GetPCLKFrequency(CGU_PERIPHERAL_M4CORE)/1.0E6));
	printf("CGU_PERIPHERAL_EMC is clocked at %1.1f MHz\r\n", ((float)CGU_GetPCLKFrequency(CGU_PERIPHERAL_EMC)/1.0E6));
		
	
	LED_GPIO_Configuration();
	
	UART_Configuration();
	/* Change the clock to 204 MHz */
	/* Set PL160M @ 12*17=204 MHz */
// 	if(CGU_SetPLL1(17) != CGU_ERROR_SUCCESS) while(1);
// 	CGU_UpdateClock();
// 	


	/* initialize memory */
	printf("Init NAND Flash...\r\n");
	NandFlash_Init();
	
	printf("RESET NAND Flash...\r\n");
	NandFlash_Reset();

	FlashID = NandFlash_ReadId();
	printf("NAND Flash ID:0X%X!\r\n",FlashID);
    printf("Checking valid block...\r\n");
    if ( NandFlash_ValidBlockCheck() == FALSE )
    {
    	printf("Valid block checking error, testing terminated! At block(s): ");
 //   	while ( 1 );		// Fatal error

			for(i = 0; i < NANDFLASH_NUMOF_BLOCK; i++)
			{
				if (InvalidBlockTable[i] == FALSE)
				{
					printf("0X%X",i);
				}
			}
    }
		
    else
    printf("Valid block checking OK\r\n");
		
    /**************************************************************
    * NandFlash_BlockErase() could scrub off all the invalid 	    *
    * block infomation including the factory initial invalid	    *
    * block table information. Per Samsung's K9F1G08 Users Manual,*
    * "Any intentional erasure of the initial invalid block		*
    * information is prohibited.									*
    *	However, during the driver debugging, it may create lot of  *
    *	invalid blocks. Below NandFlash_BlockErase() is used to deal *
    * with situation like that.                                   *
    *																*
    ***************************************************************/
    /* Erase the entire NAND FLASH */
    printf("Erase entire NAND Flash...\r\n");
    for ( i = 0; i < NANDFLASH_NUMOF_BLOCK; i++ )
    {
	  	if ( NandFlash_BlockErase(i) == FALSE )
	  	{
			printf("Erase NAND Flash fail at block: 0X%X",i);
	  	}
    }
    /* For the test program, the pattern for the whole page 2048 bytes
    is organized as: 0x0, 0x1, ... 0xFF, 0x0, 0x01...... */
		
		printf("Write a block of 2K data to NAND Flash...\r\n");		
    for ( i = 0; i < NANDFLASH_RW_PAGE_SIZE; i++ )
    {
	  	ReadBuf[i] = 0;
	  	WriteBuf[i] = i;
//			printf(" 0x%x",i);
    }
    /* If it's a valid block, program all the pages of this block,
    read back, and finally validate. */
    /* simple program to program the block 0 and page 0 */
    if ( InvalidBlockTable[0] == 0 )
    {    	
	  	if ( NandFlash_PageProgram( 0, 0, &WriteBuf[0] ) == FALSE )
		  	{
	  			printf("Writing fail, testing terminated!\r\n");
	  			while ( 1 );	/* Fatal error */
		  	}

	  	printf("Read back a block of 2K data from NAND Flash...\r\n\r\n\r\n");
	  	if ( NandFlash_PageRead( 0, 0, &ReadBuf[0] ) == FALSE )
		  	{
	  			printf("Reading fail, testing terminated!\r\n");
	  			while ( 1 );	/* Fatal error */
		  	}
			printf("Read a block of 2K data to NAND Flash...\r\n");		
			for ( i = 0; i < NANDFLASH_RW_PAGE_SIZE; i++ )
			{
				ReadBuf[i] = i;
//				printf(" 0x%x",i);
			}
	  	/* Comparison read and write buffer */
	  	printf("Verify data...\r\n");
	  	for ( i = 0; i < NANDFLASH_RW_PAGE_SIZE; i++ )
	  	{
			if ( ReadBuf[i] != WriteBuf[i] )
			{
			  	printf("Verifying fail, testing terminated!\r\n");
				while ( 1 );	/* Fatal error */
			}
	  	}
    }

    printf("Verifying complete! Testing terminated!");
	
	while (1)
	{
	GPIO_ClearValue(LED1_GPIO_PORT,(1<<LED1_GPIO_BIT));	
	GPIO_ClearValue(LED2_GPIO_PORT,(1<<LED2_GPIO_BIT));	
	GPIO_ClearValue(LED3_GPIO_PORT,(1<<LED3_GPIO_BIT));	
	GPIO_ClearValue(LED4_GPIO_PORT,(1<<LED4_GPIO_BIT));	
		timer_delay_ms(100);
		GPIO_SetValue(LED1_GPIO_PORT,(1<<LED1_GPIO_BIT));
		GPIO_SetValue(LED2_GPIO_PORT,(1<<LED2_GPIO_BIT));
		GPIO_SetValue(LED3_GPIO_PORT,(1<<LED3_GPIO_BIT));
		GPIO_SetValue(LED4_GPIO_PORT,(1<<LED4_GPIO_BIT));
		timer_delay_ms(100);
	//	printf("\r\nWelcome Using Open4330 development \r\n");
	}

}

#ifdef  DEBUG
/*******************************************************************************
* @brief		Reports the name of the source file and the source line number
* 				where the CHECK_PARAM error has occurred.
* @param[in]	file Pointer to the source file name
* @param[in]    line assert_param error line source number
* @return		None
*******************************************************************************/
void check_failed(uint8_t *file, uint32_t line)
{
	/* User can add his own implementation to report the file name and line number,
	 ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

	/* Infinite loop */
	while(1);
}
#endif

/**
 * @}
 */
