/**********************************************************************
* $Id$		Gpio_LedBlinky.c	2011-06-02
*//**
* @file		Gpio_LedBlinky.c
* @brief	This example describes how to use GPIO to drive LEDs
* @version	1.0
* @date		02. June. 2011
* @author	NXP MCU SW Application Team
*
* Copyright(C) 2011, NXP Semiconductor
* All rights reserved.
*
***********************************************************************
* Software that is described herein is for illustrative purposes only
* which provides customers with programming information regarding the
* products. This software is supplied "AS IS" without any warranties.
* NXP Semiconductors assumes no responsibility or liability for the
* use of the software, conveys no license or title under any patent,
* copyright, or mask work right to the product. NXP Semiconductors
* reserves the right to make changes in the software without
* notification. NXP Semiconductors also make no representation or
* warranty that such application will be suitable for the specified
* use without further testing or modification.
**********************************************************************/
#include "lpc43xx_gpio.h"
#include "lpc43xx_scu.h"
#include "lpc43xx_libcfg_default.h"
#include "lpc43xx_cgu.h"
#include "LPC4357_IO.h"
#include "lpc4357_uart.h"
#include "lpc43xx_rtc.h"
#include "lpc43xx_evrt.h"

//#include "lpc43xx_utils.h"


/* Example group ----------------------------------------------------------- */
/** @defgroup Gpio_LedBlinky	Gpio_LedBlinky
`	
 * @ingroup GPIO_Examples
 * @{
 */

void RTC_Config(void);

/*----------------- INTERRUPT SERVICE ROUTINES --------------------------*/
/*********************************************************************//**
 * @brief		RTC interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void RTC_IRQHandler(void)
{
	
	uint32_t minval;

	/* This is increment counter interrupt*/
	if (RTC_GetIntPending(LPC_RTC, RTC_INT_COUNTER_INCREASE))
	{
		// Clear pending interrupt
		RTC_ClearIntPending(LPC_RTC, RTC_INT_COUNTER_INCREASE);

		minval = RTC_GetTime (LPC_RTC, RTC_TIMETYPE_MINUTE);
		/* Send debug information */
		printf("Minute: %d",minval);
	}

	/* Continue to check the Alarm match*/
	if (RTC_GetIntPending(LPC_RTC, RTC_INT_ALARM))
	{
		// Clear pending interrupt
		RTC_ClearIntPending(LPC_RTC, RTC_INT_ALARM);

		/* Send debug information */
		printf ("ALARM 30s matched!");
	}

}

/*********************************************************************//**
 * @brief		EVRT interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void EVRT_IRQHandler(void)
{
	if(EVRT_IsSourceInterrupting(LPC_EVENTROUTER,EVRT_SRC_RTC))
	{
		EVRT_ClrPendIntSrc(LPC_EVENTROUTER,EVRT_SRC_RTC);
		RTC_IRQHandler();
	}
}


/* With ARM and GHS toolsets, the entry point is main() - this will
   allow the linker to generate wrapper code to setup stacks, allocate
   heap area, and initialize and copy code and data segments. For GNU
   toolsets, the entry point is through __start() in the crt0_gnu.asm
   file, and that startup code will setup stacks and data */
int main(void)
{
	uint32_t hour,min,sec,pre_sec;	
  SystemInit();
	CGU_Init();

	/* Change the clock to 204 MHz */
	/* Set PL160M @ 12*17=204 MHz */
// 	if(CGU_SetPLL1(17) != CGU_ERROR_SUCCESS) while(1);
// 	CGU_UpdateClock();
// 	CGU_ConfigPWR(CGU_PERIPHERAL_GPIO,ENABLE);
// 	
	UART_Configuration();
	
	printf("Core is clocked at %1.1f MHz\r\n", ((float)CGU_GetPCLKFrequency(CGU_PERIPHERAL_M4CORE)/1.0E6));
	RTC_Config();
	sec=pre_sec=0;	
   while(1)
	{
		
		hour =  RTC_GetTime(LPC_RTC, RTC_TIMETYPE_HOUR);
		min =  RTC_GetTime(LPC_RTC, RTC_TIMETYPE_MINUTE);
		sec =  RTC_GetTime(LPC_RTC, RTC_TIMETYPE_SECOND);
		if(sec!=pre_sec)
		{
		 	pre_sec = sec;
			printf("Current time :%2d:%2d:%2d \r\n",hour,min,sec);
		}
	}
}

void RTC_Config(void)
{
	
	EVRT_SetUpIntSrc(LPC_EVENTROUTER,EVRT_SRC_RTC,DISABLE);
	/* RTC Block section ------------------------------------------------------ */
	// Init RTC module
	RTC_Init(LPC_RTC);

	/* Set current time for RTC */
	// Current time is 0:0:00PM, 2013-03-21
	RTC_SetTime (LPC_RTC, RTC_TIMETYPE_SECOND, 0);
	RTC_SetTime (LPC_RTC, RTC_TIMETYPE_MINUTE, 0);
	RTC_SetTime (LPC_RTC, RTC_TIMETYPE_HOUR, 0);
	RTC_SetTime (LPC_RTC, RTC_TIMETYPE_MONTH, 3);
	RTC_SetTime (LPC_RTC, RTC_TIMETYPE_YEAR, 2013);
	RTC_SetTime (LPC_RTC, RTC_TIMETYPE_DAYOFMONTH, 21);

	/* Set ALARM time for second */
	RTC_SetAlarmTime (LPC_RTC, RTC_TIMETYPE_SECOND, 30);

	/* Set the AMR for 30s match alarm interrupt */
	RTC_AlarmIntConfig (LPC_RTC, RTC_TIMETYPE_SECOND, ENABLE);

	/* Set the CIIR for minute counter interrupt*/
	RTC_CntIncrIntConfig (LPC_RTC, RTC_TIMETYPE_MINUTE, ENABLE);

	/* EVRT Block section ------------------------------------------------------ */
	EVRT_Init(LPC_EVENTROUTER);
	/* Configure RTC signal to EVRT is rising edge*/
	EVRT_ConfigIntSrcActiveType(LPC_EVENTROUTER,EVRT_SRC_RTC,EVRT_SRC_ACTIVE_RISING_EDGE);
	/* Mount RTC interrupt to EVRT*/
	EVRT_SetUpIntSrc(LPC_EVENTROUTER,EVRT_SRC_RTC,ENABLE);
	/* Clear pending interrupt from RTC if any*/
	EVRT_ClrPendIntSrc(LPC_EVENTROUTER,EVRT_SRC_RTC);

    /* Disable EVRT interrupt */
    NVIC_DisableIRQ(EVENTROUTER_IRQn);
    /* preemption = 1, sub-priority = 1 */
    NVIC_SetPriority(EVENTROUTER_IRQn, ((0x01<<3)|0x01));
    /* Enable RTC interrupt */
    NVIC_EnableIRQ(EVENTROUTER_IRQn);

	/* Enable rtc (starts increase the tick counter and second counter register) */
	RTC_Cmd(LPC_RTC, ENABLE);
	
}


#ifdef  DEBUG
/*******************************************************************************
* @brief		Reports the name of the source file and the source line number
* 				where the CHECK_PARAM error has occurred.
* @param[in]	file Pointer to the source file name
* @param[in]    line assert_param error line source number
* @return		None
*******************************************************************************/
void check_failed(uint8_t *file, uint32_t line)
{
	/* User can add his own implementation to report the file name and line number,
	 ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

	/* Infinite loop */
	while(1);
}
#endif

/**
 * @}
 */
